@@ -11,7 +11,7 @@ from collections import OrderedDict
 import numpy as np
 import pandas as pd
 
-from bric_analysis_libraries import standard_functions as std
+from .. import standard_functions as std
 
 
 # Helper functions
