@@ -7,19 +7,132 @@
 import numpy as np
 import pandas as pd
 import matplotlib as mpl
+import matplotlib.cm as cm
 import matplotlib.pyplot as plt
 
 
-def set_plot_defaults():
-    """
-    Set matplotlib plotting defautls
-    """
+# @todo: use dataclass
+"""
+Base font
+"""
+font_base = {
+    'figure': { 'figsize': ( 10, 8 ) }
+}
 
-    # set plot format defaults
-    mpl.rc( 'font', size = 16 )
-    mpl.rc( 'xtick', labelsize = 14 )
-    mpl.rc( 'ytick', labelsize = 14 )
-    mpl.rc( 'figure', figsize = ( 10, 8 ) )
+"""
+Default font.
+Use for data exploration.
+"""
+font_def = {
+    **font_base,
+    'font': { 'size': 16 },
+    'xtick': { 'labelsize': 14 },
+    'ytick': { 'labelsize': 14 },
+}
+
+
+"""
+Publication font.
+Use for publication figures.
+Enlarged font for better readability on paper.
+"""
+font_pub = {
+    **font_base,
+    'font': { 'size': 24 },
+    'xtick': { 'labelsize': 18 },
+    'ytick': { 'labelsize': 18 },
+    'axes': { 'labelpad': 15, 'titlepad': 15 },
+}
+
+
+def set_plot_defaults( font = font_def ):
+    """
+    Set matplotlib plotting defaults.
+
+    :param font: Dictionary representing the font to set.
+        Keys should be the matplotlib.pyplot.rcParams.group.
+        Values are a dictionary of the arguments to set for the group,
+        keyed by argument name with values the value to set.
+        [Default: bric_analysis_libraries.plot.font_def]
+    """
+    for group, args in font.items():
+        mpl.rc( group, **args )
+
+
+def get_scaled_colormap( name, min_val = 0, max_val = 1, raise_err = False ):
+    """
+    :param name: Name of the color map.
+    :param min_val: Minimum value of range. [Default: 0]
+    :param max_val: Maximum value of range. [Default: 1]
+    :param raise_err: Raise an error if value is outside bounds.
+        If False, clips to range.
+        [Default: False]
+    :returns: Function that accepts an argument of the value,
+        and returns the corresponding color value.
+    """
+    if min_val >= max_val:
+        raise ValueError( '`min_val` is less or equal to `max_val`.' )
+
+    cmap = cm.get_cmap( name )
+    val_range = max_val - min_val
+    def _scaled_map( val ):
+        """
+        Converts a value in a range to a color value.
+
+        :param val: Value or iterable of values to convert to a color.
+        :returns: ( r, g, b, a ) or list of ( r, g, b, a ) color values.
+        """
+        if not( isinstance( val, int ) or isinstance( val, float ) ):
+            val = np.array( val )
+
+        norm_val = ( val - min_val )/ val_range
+        if raise_err:
+            if ( norm_val < 0 ) or ( norm_val > 1 ):
+                raise ValueError( 'Value outside bounds.' )
+
+        np.clip( norm_val, a_min = 0, a_max = 1 )
+        return cmap( norm_val )
+
+
+    return _scaled_map
+
+
+def fig_for_publication( fig, font = None, label_map = None ):
+    """
+    Set a figures fonts for publication.
+
+    :param fig: Figure to prepare.
+    :param font: Dictionary of font values.
+        Only keys that should be changes from the default should be included.
+        + major_label_size: Font size of the major labels. [Default: 24]
+        + major_labelpad: Padding of the major labels. [Default: 15]
+        + major_tick_size: Font size of the major tick labels. [Default: 18]
+    :param label_map: Dictionary of labels and their mapped values. [Default: {}]
+    """
+    if font is None:
+        font = {}
+
+    if label_map is None:
+        label_map = {}
+
+    defaults = {
+        'major_label_size': 24,
+        'major_labelpad': 15,
+        'major_tick_size': 18
+    }
+
+    font = { **defaults, **font }
+
+    for ax in fig.axes:
+        saxs = [ ax.xaxis, ax.yaxis ]
+        for sax in saxs:
+            lbl_text = sax.label.get_text()
+            if lbl_text in label_map:
+                sax.label.set_text( label_map[ lbl_text ] )
+
+            sax.label.set_fontsize( font[ 'major_label_size' ] )
+            sax.labelpad = font[ 'major_labelpad' ]
+            sax.set_tick_params( labelsize = font[ 'major_tick_size' ] )
 
 
 def save_figure( path, kind = 'png', fig = None ):
@@ -31,7 +144,6 @@ def save_figure( path, kind = 'png', fig = None ):
     :param fig: Figure to save. If None, saves current figure.
         [Default: None]
     """
-
     if fig is None:
         fig = plt.gcf()
 
@@ -46,7 +158,13 @@ def index_from_counter( counter, rows, cols ):
     :param rows: Number of rows in matrix.
     :param cols: Number of columns in matrix.
     :returns: ( row, column ) of counter.
+    :raises: ValueError if `counter` is invalid.
     """
+    if counter < 0:
+        raise ValueError( 'Counter must be a non-negative integer.' )
+
+    if counter > rows* cols:
+        raise ValueError( 'Counter exceeds total grid size.' )
 
     row = int( np.floor( counter / cols ) )
     col = int( counter % cols )
@@ -54,15 +172,18 @@ def index_from_counter( counter, rows, cols ):
     return ( row, col )
 
 
-def ax_from_counter( counter, axs ):
+def ax_from_counter( counter, axs, reverse = False ):
     """
     Gets an axis from an array of axes based on a counter.
 
     :param counter: Counter.
     :param axs: Matrix of axes.
+    :param reverse: Reverse order of axes. [Defualt: False]
     :returns: Axis.
     """
     row, col = index_from_counter( counter, *axs.shape )
+    if reverse:
+        row = axs.shape[ 0 ] - 1 - row
 
     if len( axs.shape ) == 1:
         # only rows
@@ -75,31 +196,101 @@ def ax_from_counter( counter, axs ):
     return ax
 
 
-def rows_needed( df, cols = 1, level = None ):
+def rows_needed( df, cols = 1, level = None, axis = 1 ):
     """
     Returns the number of rows needed for a plot, givent the number of columns.
 
-    :param df: DataFrame or DataFrameGroupBy.
+    :param df: DataFrame, DataFrameGroupBy, Series, or SeriesGroupBy.
     :param cols: Number of columns. [Default: 1]
     :param level: DataFrame level to use for grouping,
         or None to count each column.
-        If `df` is a DataFrameGroupBy, this does not have any effect.
+        If `df` is a DataFrameGroupBy or SeriesGroupBy this does not have an effect.
         [Default: None]
+    :param axis: Axis to determine rows from.
+        Not applicable if `df` is not DataFrame.
+        [Default: 1]
+    :returns: Number of rows needed for iterating over the data given the number of columns.
     """
-    if isinstance( df, pd.core.groupby.DataFrameGroupBy ):
+    if (
+        isinstance( df, pd.core.groupby.DataFrameGroupBy ) or
+        isinstance( df, pd.core.groupby.SeriesGroupBy )
+    ):
         num_plots = len( df )
 
-    else:
+    elif (
+        isinstance( df, pd.DataFrame ) or
+        isinstance( df, pd.Series )
+    ):
+        if isinstance( df, pd.Series ):
+            axis = 0
+
         if level is not None:
-            num_plots = df.columns.get_level_values( level ).unique().shape[ 0 ]
+            num_plots = df.axes[ axis ].get_level_values( level ).unique().shape[ 0 ]
 
         else:
-            num_plots = df.columns.shape[ 0 ]
+            num_plots = df.axes[ axis ].shape[ 0 ]
+
+    else:
+        # invalid type
+        raise TypeError( 'Invalid `df` object. Must be a pandas DataFrame, DataFrameGroupBy, Series, or SeriesGroupBy.' )
 
     return int( np.ceil( num_plots/ cols ) )
 
 
-def plot_levels( plot, df, show = True, level = 'metrics', axis = 1, **fig_args ):
+def rows_cols_square( df, level = None, axis = 1 ):
+    """
+    Returns the number of rows and columns needed to
+    create a square set of axes for the data.
+
+    :param df: DataFrame, DataFrameGroupBy, Series, or SeriesGroupBy.
+    :param level: DataFrame level to use for grouping,
+        or None to count each column.
+        If `df` is a DataFrameGroupBy or SeriesGroupBy this does not have an effect.
+        [Default: None]
+    :param axis: Axis to determine rows from.
+        Not applicable if `df` is not DataFrame.
+        [Default: 1]
+    :returns: Tuple of ( rows, cols ) needed for iterating over the data.
+    """
+    if (
+        isinstance( df, pd.core.groupby.DataFrameGroupBy ) or
+        isinstance( df, pd.core.groupby.SeriesGroupBy )
+    ):
+        num_plots = len( df )
+
+    elif (
+        isinstance( df, pd.DataFrame ) or
+        isinstance( df, pd.Series )
+    ):
+        if isinstance( df, pd.Series ):
+            axis = 0
+
+        if level is not None:
+            num_plots = df.axes[ axis ].get_level_values( level ).unique().shape[ 0 ]
+
+        else:
+            num_plots = df.axes[ axis ].shape[ 0 ]
+
+    else:
+        # invalid type
+        raise TypeError( 'Invalid `df` object. Must be a pandas DataFrame, DataFrameGroupBy, Series, or SeriesGroupBy.' )
+
+    sqrt = num_plots**( 1/2 )
+    rows = int( np.ceil( sqrt ) )
+    cols = int( np.round( sqrt ) )
+
+    return ( rows, cols )
+
+
+def plot_levels(
+    plot,
+    df,
+    show = True,
+    level = 'metrics',
+    axis = 1,
+    flip_axes = False,
+    **fig_args
+    ):
     """
     Plots each element of a Pandas DataFrame in a separate subplot.
 
@@ -108,6 +299,8 @@ def plot_levels( plot, df, show = True, level = 'metrics', axis = 1, **fig_args
     :param show: Show the plot. [Defualt: True]
     :param level: Which level to iterate over. [Default: 'metrics']
     :param axis: The axis to iterate over. [Default: 'columns']
+    :param flip_axes: Flip the order of the axes, plotting from the bottom first.
+        [Default: False]
     :param fig_args: Keyword arguments passed to plt.subplot().
     :returns: The Figure and Axes of the plot as a tuple ( fig, axs ).
     """
@@ -129,7 +322,7 @@ def plot_levels( plot, df, show = True, level = 'metrics', axis = 1, **fig_args
     index = 0
 
     for name, data in groups:
-        ax = ax_from_counter( index, axs )
+        ax = ax_from_counter( index, axs, reverse = flip_axes )
         plot( ax, data, name )
         index += 1
 
@@ -225,10 +418,12 @@ def temperature_plot_rainbow(
     :param colorbar: Whether to include the color bar legend. [Default: True]
     :param level: Index or name of the level used for coloring. [Default: 0]
     :param ax: Axis to plot on. If None, creates one. [Default: None]
-    :param color_by_order: Color traces by order of values, rather than actual value.
+    :param color_by_order: Color traces by order of values rather than actual value.
         [Default: False]
     :param cmap: Colormap to use. [Defualt: 'jet']
     :param **kwargs: Arguments passed to pandas.DataFrame#plot
+    :returns: Tuple of ( fig, ax, None ) if colorbar is False,
+        or ( fig, axs, cbar ) if colorbar is True.
     """
     is_df = isinstance( df, pd.DataFrame )
     if ax is None:
@@ -246,7 +441,11 @@ def temperature_plot_rainbow(
     val_min = temp_vals.min()
     val_max = temp_vals.max()
     
-    cm = plt.get_cmap( cmap )
+    cm = (
+        plt.get_cmap( cmap )
+        if isinstance( cmap, str ) else
+        cmap
+    )
 
     if color_by_order:
         # color by vlaue index
@@ -264,7 +463,7 @@ def temperature_plot_rainbow(
     else:
         # color by value
         colors = [
-            cm( ( temp_val - val_min )/( val_max - val_min ) )  # normalize values sbetween 0 and 1
+            cm( ( temp_val - val_min )/( val_max - val_min ) )  # normalize values between 0 and 1
             for temp_val in temp_vals
         ]
 
@@ -275,11 +474,11 @@ def temperature_plot_rainbow(
         cbar = mpl.colorbar.ColorbarBase(
             ax = cax,
             cmap = cm,
+            orientation = 'vertical',
             norm = mpl.colors.Normalize(
                 vmin = val_min,
                 vmax = val_max
-            ),
-            orientation = 'vertical'
+            )
         )
 
         if is_df:
@@ -298,7 +497,67 @@ def temperature_plot_rainbow(
 
         cbar.set_label( cbar_label, labelpad = 15 )
 
-    return ( fig, ax )
+    if colorbar:
+        return ( fig, ax, cbar )
+
+    else:
+        return ( fig, ax, None )
+
+
+def color_plot_2d(
+    df,
+    shading = 'nearest',
+    colorbar = True,
+    ax = None,
+    **kwargs
+):
+    """
+    Creates a 2D color plot from a DataFrame.
+    The DataFrame must have a pandas.index for both
+    the index and columns, i.e. non pandas.MultiIndex's.
+    The columns map to the x-axis, and the index to the y-axis.
+
+    :param df: DataFrame.
+    :param shading: Shading to use.
+        Values are [ 'nearest', 'gouraud' ].
+        [Default: 'nearest']
+    :param colorbar: Include colorbar. [Defualt: True]
+    :param ax: Axis to plot on.
+        If None creates a new axis.
+        [Default: None]
+    :param **kwargs: Args passed to the plotting function.
+    :returns: ( fig, ax, cbar, img )
+        If `colorbar` is False, cbar is None.
+    :raises TypeError: If DataFrame's index or columns are not a pandas.Index.
+    :raises ValueError: If `shading` is not valid. 
+    """
+    if not(
+        isinstance( df.index, pd.Index ) and
+        isinstance( df.columns, pd.Index )
+    ):
+        raise TypeError( 'Both index and columns must be pandas.Index, no MultiIndexes are allowed.' )
+
+    if shading not in [ 'nearest', 'gouraud' ]:
+        raise ValueError( 'Invalid shading value. Must be in [ "nearest", "gouraud" ].' )
+
+    cbar = None
+    x, y = np.meshgrid( df.columns, df.index )
+
+    if ax is not None:
+        fig = ax.get_figure()
+    
+    else:    
+        fig, ax = plt.subplots()
+    
+    img = ax.pcolormesh( x, y, df.values, shading = shading, **kwargs )
+    
+    if colorbar:
+        cbar = fig.colorbar( img, ax = ax )
+    
+    ax.set_xlabel( df.columns.name )
+    ax.set_ylabel( df.index.name )
+
+    return ( fig, ax, cbar, img )
 
 
 def outer_plot(
